// included from solver.rs

// This will take as many packages as possible.
fn parse_carry_skip_finish<'m>(
    g: &Globals,
    m: &mut MutGlobals,
    mut layer: &'m[LayerVocab],
) -> Option<(Mindset, &'m[LayerVocab])> {
    let mut result = None;

    while let Some((ms, new_layer)) = subparse_carry_skip_finish(g, m, layer) {
        result = Some((ms, new_layer));
        layer = new_layer;
    }

    result
}

// This handles one package only.
fn subparse_carry_skip_finish<'m>(
    g: &Globals,
    m: &mut MutGlobals,
    mut layer: &'m[LayerVocab],
) -> Option<(Mindset, &'m[LayerVocab])> {

    let mut width_taken = 0;            // # addends = # columns in this case.
    let mut baggage_taken = 0;          // cumulative propagates & carries
    let mut base_col = 0;               // This value is impossible.
    let mut stage_widths = None;        // Skip alloc until need certain.

    // See if we have LayerVocab indicating a CarrySkipFinish.
    while let [
        FinishInfo { prop_carry_baggage, .. }, 
        FinishWithWidth { gpv, width },
    ..] = layer {

        // If a package has enough space, it can finish multiple packages
        // from the north. So see if there's still space.
        if width_taken + width + prop_carry_baggage > g.ram_dims.0
                || width_taken + width > g.ram_dims.1 {
            assert!(base_col != 0);     // This would be a logic error north.
            break;                      // This would exceed package capacity.
        }

        // We commit here to include these in our package.
        layer = &layer[2..];
        width_taken += width;
        baggage_taken = *prop_carry_baggage;
        if base_col == 0 {
            base_col = *gpv;
        }
        if stage_widths == None {
            stage_widths = Some(Vec::new());
        }
        if let Some(ref mut v) = stage_widths {
            v.push(*width as u64);      // sic. u64
        }
    }

    if base_col == 0 {
        return None;                    // No CarrySkipFinish here.
    }

    // Emit the package.
    let pkg = Package {
        kind: CarrySkipFinish { stage_widths: stage_widths.unwrap() },
        mem_used: width_taken << width_taken + baggage_taken,
        output_pvs: (base_col, base_col + width_taken),
        input_pvs: (base_col..base_col + width_taken).collect(),
    };
    m.trial_soln.push(pkg);

    // Put this CarrySkipFinish's output bits into the next layer.
    m.overlap_arbgeom_output(base_col, base_col + width_taken);

    // Return a new mindset, and new position.
    Some((NeverMind, layer))
}
