// included from solver.rs

// parse_wireflop() will take as many columns as possible.
fn parse_wireflop<'m>(
    m: &mut MutGlobals,
    ms: Mindset,
    mut layer: &'m[LayerVocab],
) -> Option<(Mindset, &'m[LayerVocab])> {

    // Must start with a Column containing exactly one (untaken) bit.
    let (gpv, n) = if let [Column { gpv, n }, ..] = layer {
        (*gpv, *n - ms.missing_bits())
    } else {
        return None;
    };

    if n != 1 {
        return None;
    }

    // So far, we have a one-column wireflop.
    let mut cols = 1;
    layer = &layer[1..];

    // Get further one-bit columns, if any.
    while let [Column { n: 1, .. }, ..] = layer {
        cols += 1;
        layer = &layer[1..];
    }

    // Emit the package.
    let pkg = Package {
        kind: Wireflop,
        mem_used: 0,
        output_pvs: (gpv, gpv + cols),
        input_pvs: (gpv..gpv + cols).collect(),
    };
    m.trial_soln.push(pkg);

    // Put this Wireflop's output bits into the next layer.
    // It's possible the first bit will overlap a prior ArbGeom.
    m.overlap_arbgeom_output(gpv, gpv + cols);

    // Return a new mindset, and new position.
    Some((NeverMind, layer))
}
