///! urand64.rs February 2022
///! This module provides random number generation for regression tests
///! via BigRand, but it doesn't work under Microsoft Windows due to
///! the absence of /dev/urandom on that platform. XXX

use std::io::prelude::*;
use std::io::BufReader;     // for Urand64
use std::fs::File;          // for Urand64

use crate::*;

/// This is a simple gadget for pulling u64s from /dev/urandom.
pub struct Urand64 {
    reader: Option<BufReader<File>>,
}

impl Urand64 {

    /// Creates a new Urand64. This function always succeeds,
    /// but may not succeed correctly.
    pub fn new() -> Urand64 {

        let f = File::open("/dev/urandom");
        if let Err(_) = f {
            return Urand64 { reader: None };
        }

        let reader = BufReader::new(f.unwrap());
        return Urand64 { reader: Some(reader) };
    }

    /// Grab another 8-byte integer from /dev/urandom.
    /// This function terminates the program if it doesn't succeed,
    /// which isn't an ideal model for distribution.
    pub fn next(&mut self) -> u64 {
        if let None = self.reader {
            fail("/dev/urandom not open");
        }

        let mut buf = [0; 8];
        match self.reader.as_mut().unwrap().read_exact(&mut buf) {
            Err(_) => fail("/dev/urandom read failure"),
            Ok(()) => u64::from_ne_bytes(buf)
        }
    }
}

/// This test function displays a little PRNG output.
pub fn test_urand64() {
    let mut u = Urand64::new();
    p!(Y "test_urand64 output:\n");
    for _ in 0..10 {
        for _ in 0..4 {
            pp!("{:016x} ", u.next());
        }
        p!();
    }
    p!();
}
